-- Create new Module
local Avion_Aggro 	= Avion_Core:NewModule("Avion_Aggro")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Aggro")


-- Module information, default options and name
Avion_Aggro.name 	= "Avion_Aggro"


-- Register SavedVariables
function Avion_Aggro:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Aggro", "profile", { 
		Banzai_PlayerGainedAggro = { 
			animationIDList = { },
			enabled = true,
		},
		Banzai_PlayerLostAggro = { 
			animationIDList = { },
			enabled = true,
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Aggro")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Aggro:UpdateEventRegistration(unLoad)
	local data = self.db.profile

	if (not unLoad and data.Banzai_PlayerGainedAggro.enabled) then
		self:RegisterEvent("Banzai_PlayerGainedAggro")
	else
		if (self:IsEventRegistered("Banzai_PlayerGainedAggro")) then
			self:UnregisterEvent("Banzai_PlayerGainedAggro")
		end
	end
	
	if (not unLoad and data.Banzai_PlayerLostAggro.enabled) then
		self:RegisterEvent("Banzai_PlayerLostAggro")
	else
		if (self:IsEventRegistered("Banzai_PlayerLostAggro")) then
			self:UnregisterEvent("Banzai_PlayerLostAggro")
		end
	end
end


-- Build Optionstable
function Avion_Aggro:BuildOptionTable()
	local data = self.db.profile

	-- AceOptionTable
	local aggroOptions = {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			Banzai_PlayerGainedAggro = {
				order = 1,
				type = 'group',
				name = L["Banzai_PlayerGainedAggro"],
				desc = L["Banzai_PlayerGainedAggro_Desc"],
				args = { 
					enabled = {
						order = 1,
						type = 'toggle',
						name = L["Enabled"],
						desc = L["Enabled_Desc"],
						get = function()
							return data.Banzai_PlayerGainedAggro.enabled
						end,
						set = function(toggle)
							data.Banzai_PlayerGainedAggro.enabled = toggle
							
							self:UpdateEventRegistration(false)
						end,
					},
					list = {
						order = 2,
						type = 'group',
						name = L["AnimationList"],
						desc = L["AnimationList_Desc"],
						args = Avion_Core:BuildAnimationList(data.Banzai_PlayerGainedAggro.animationIDList),
					},
				},
			},
			Banzai_PlayerLostAggro = {
				order = 1,
				type = 'group',
				name = L["Banzai_PlayerLostAggro"],
				desc = L["Banzai_PlayerLostAggro_Desc"],
				args = { 
					enabled = {
						order = 1,
						type = 'toggle',
						name = L["Enabled"],
						desc = L["Enabled_Desc"],
						get = function()
							return data.Banzai_PlayerLostAggro.enabled
						end,
						set = function(toggle)
							data.Banzai_PlayerLostAggro.enabled = toggle
							
							self:UpdateEventRegistration(false)
						end,
					},
					list = {
						order = 2,
						type = 'group',
						name = L["AnimationList"],
						desc = L["AnimationList_Desc"],
						args = Avion_Core:BuildAnimationList(data.Banzai_PlayerLostAggro.animationIDList),
					},
				},
			},
		}
	}
	
	return aggroOptions
end


-- Save and return
function Avion_Aggro:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Aggro:AnimationDeleted(animationID)
	if (self.db.profile.Banzai_PlayerGainedAggro.animationIDList[animationID]) then
		self.db.profile.Banzai_PlayerGainedAggro.animationIDList[animationID] = nil
	end
	
	if (self.db.profile.Banzai_PlayerLostAggro.animationIDList[animationID]) then
		self.db.profile.Banzai_PlayerLostAggro.animationIDList[animationID] = nil
	end
end


-- Player gained aggro, trigger animation
function Avion_Aggro:Banzai_PlayerGainedAggro(unitTable)
	Avion_Core:TriggerAnimationsByID("GainedAggro", self.db.profile.Banzai_PlayerGainedAggro.animationIDList)
end


-- Player lost aggro, trigger animation
function Avion_Aggro:Banzai_PlayerLostAggro(unitTable)
	Avion_Core:TriggerAnimationsByID("LostAggro", self.db.profile.Banzai_PlayerLostAggro.animationIDList)
end


