-- If not found don't do anything
if (not BigWigs) then
	return
end

-- Create new Module
local Avion_BigWigs 	= Avion_Core:NewModule("Avion_BigWigs")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_BigWigs")


-- Module information, default options and name
Avion_BigWigs.name 	= "Avion_BigWigs"


-- Register SavedVariables
function Avion_BigWigs:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_BigWigs", "profile", { 
		BigWigs_Message = {
		},
		BigWigs_StartBar = {
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_BigWigs")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_BigWigs:UpdateEventRegistration(unLoad)
	local isEnabled
	
	isEnabled = false
	for _, data in pairs(self.db.profile.BigWigs_Message) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and isEnabled) then
		self:RegisterEvent("BigWigs_Message")
	else
		if (self:IsEventRegistered("BigWigs_Message")) then
			self:UnregisterEvent("BigWigs_Message")
		end
	end
	
	isEnabled = false
	for _, data in pairs(self.db.profile.BigWigs_StartBar) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end
	
	if (not unLoad and isEnabled) then
		self:RegisterEvent("BigWigs_StartBar")
	else
		if (self:IsEventRegistered("BigWigs_StartBar")) then
			self:UnregisterEvent("BigWigs_StartBar")
		end
	end
end


-- Build OptioTable for certain Message
function Avion_BigWigs:BuildOptionTable_BigWigs_Message(text)
	return {
		type = 'group',
		name = L["Text"] .. text,
		desc = L["Text_Desc"],
		args = { 
			enabled = {
				order = 1,
				type = 'toggle',
				name = L["Enabled"],
				desc = L["Enabled_Desc"],
				get = function()
					return self.db.profile.BigWigs_Message[text].enabled
				end,
				set = function(toggle)
					self.db.profile.BigWigs_Message[text].enabled = toggle
					
					self:UpdateEventRegistration(false)
				end,
			},
			list = {
				order = 2,
				type = 'group',
				name = L["AnimationList"],
				desc = L["AnimationList_Desc"],
				args = Avion_Core:BuildAnimationList(self.db.profile.BigWigs_Message[text].animationIDList),
			},
			delete = {
				order = 3,
				type = 'group',
				name = L["DeleteEntry"],
				desc = L["DeleteEntry_Desc"],
				args = {
					delete = {
						order = 1,
						type = 'execute',
						name = L["DeleteEntry"],
						desc = L["DeleteEntry_Desc"],
						func = function()
							self:DeleteBigWigs_Message(text)
						end
					},
				},
			},
		},
	}
end


-- Build OptioTable for certain bar
function Avion_BigWigs:BuildOptionTable_BigWigs_StartBar(text)
	return {
		type = 'group',
		name = L["Text"],
		desc = L["Text_Desc"],
		args = { 
			enabled = {
				order = 1,
				type = 'toggle',
				name = L["Enabled"],
				desc = L["Enabled_Desc"],
				get = function()
					return self.db.profile.BigWigs_StartBar[text].enabled
				end,
				set = function(toggle)
					self.db.profile.BigWigs_StartBar[text].enabled = toggle
					
					self:UpdateEventRegistration(false)
				end,
			},
			list = {
				order = 2,
				type = 'group',
				name = L["AnimationList"],
				desc = L["AnimationList_Desc"],
				args = Avion_Core:BuildAnimationList(self.db.profile.BigWigs_StartBar[text].animationIDList),
			},
			delete = {
				order = 3,
				type = 'group',
				name = L["DeleteEntry"],
				desc = L["DeleteEntry_Desc"],
				args = {
					delete = {
						order = 1,
						type = 'execute',
						name = L["DeleteEntry"],
						desc = L["DeleteEntry_Desc"],
						func = function()
							self:DeleteBigWigs_StartBar(text)
						end
					},
				},
			},
		},
	}
end


-- Build Optionstable
function Avion_BigWigs:BuildOptionTable()
	local BigWigs_MessageOpt = {}
	for text, _ in pairs(self.db.profile.BigWigs_Message) do
		BigWigs_MessageOpt[text] = self:BuildOptionTable_BigWigs_Message(text)
	end
	
	local BigWigs_StartBarOpt = {}
	for text, _ in pairs(self.db.profile.BigWigs_StartBar) do
		BigWigs_StartBarOpt[text] = self:BuildOptionTable_BigWigs_StartBar(text)
	end

	-- AceOptionTable
	local aggroOptions = {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'group',
				order = 1,
				name = L["Avion_NewBigWigs"],
				desc = L["Avion_NewBigWigs_Desc"],
				args = {
					BigWigs_Message = {
						type = 'text',
						order = 1,
						name = L["Avion_NewBigWigs_Message"],
						desc = L["Avion_NewBigWigs_Message_Desc"],
						usage = L["Avion_NewBigWigs_Message_Usage"],
						get = function()
					        return ""
					    end,
					    set = function(input)
					        self:NewBigWigs_Message(input)
					    end,
					    validate = function(input)
					        return (not self.db.profile.BigWigs_Message[input] and strlen(input) > 0)
					    end
					},
					BigWigs_StartBar = {
						type = 'text',
						order = 2,
						name = L["Avion_NewBigWigs_StartBar"],
						desc = L["Avion_NewBigWigs_StartBar_Desc"],
						usage = L["Avion_NewBigWigs_StartBar_Usage"],
						get = function()
					        return ""
					    end,
					    set = function(input)
					        self:NewBigWigs_StartBar(input)
					    end,
					    validate = function(input)
					        return (not self.db.profile.BigWigs_StartBar[input] and strlen(input) > 0)
					    end
					},
				},
			},
			BigWigs_Message = {
				type = 'group',
				order = 2,
				name = L["Avion_BigWigs_MessageSettings"],
				desc = L["Avion_BigWigs_MessageSettings_Desc"],
				args = BigWigs_MessageOpt,
			},
			BigWigs_StartBar = {
				type = 'group',
				order = 3,
				name = L["Avion_BigWigs_StartBarSettings"],
				desc = L["Avion_BigWigs_StartBarSettings_Desc"],
				args = BigWigs_StartBarOpt,
			},
		},
	}
	
	return aggroOptions
end


-- Save and return
function Avion_BigWigs:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Add new bar to DB
function Avion_BigWigs:NewBigWigs_StartBar(text)
	-- Insert into DB
	self.db.profile.BigWigs_StartBar[text] = {
		animationIDList = { },
		enabled = true,
	}
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Add new message to DB
function Avion_BigWigs:NewBigWigs_Message(text)
	-- Insert into DB
	self.db.profile.BigWigs_Message[text] = {
		animationIDList = { },
		enabled = true,
	}
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Delete bar from DB
function Avion_BigWigs:DeleteBigWigs_StartBar(text)
	-- Delete from DB
	self.db.profile.BigWigs_StartBar[text] = nil
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Add new message to DB
function Avion_BigWigs:DeleteBigWigs_Message(text)
	-- Delete from DB
	self.db.profile.BigWigs_Message[text] = nil
	
	-- Update option table
	Avion_Core:UpdateModuleOptionTable(self)
	
	-- Update events
	self:UpdateEventRegistration(false)
end


-- Animation was deleted, make sure its not used anymore
function Avion_BigWigs:AnimationDeleted(animationID)
	for text, data in pairs(self.db.profile.BigWigs_Message) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	for text, data in pairs(self.db.profile.BigWigs_StartBar) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
end


-- BigWigs message triggert
function Avion_BigWigs:BigWigs_Message(text) 
	-- Test if there is a trigger for this ...
	if (self.db.profile.BigWigs_Message[text] and self.db.profile.BigWigs_Message[text].enabled) then
		Avion_Core:TriggerAnimationsByID("BigWigs_Message"..text, self.db.profile.BigWigs_Message[text].animationIDList)
	end
end


-- BigWigs bar triggert
function Avion_BigWigs:BigWigs_StartBar(text) 
	-- Test if there is a trigger for this ...
	if (self.db.profile.BigWigs_StartBar[text] and self.db.profile.BigWigs_StartBar[text].enabled) then
		Avion_Core:TriggerAnimationsByID("BigWigs_StartBar"..text, self.db.profile.BigWigs_StartBar[text].animationIDList)
	end
end
