-- Create new Module
local Avion_Combo 	= Avion_Core:NewModule("Avion_Combo")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Combo")


-- Module information, default options and name
Avion_Combo.name 	= "Avion_Combo"


-- Register SavedVariables
function Avion_Combo:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Combo", "profile", { 
		ComboPoints = {
			[1] = {
				animationIDList = {	 },
				enabled = true,
			},
			[2] = {
				animationIDList = {	 },
				enabled = true,
			},
			[3] = {
				animationIDList = {	 },
				enabled = true,
			},
			[4] = {
				animationIDList = {	 },
				enabled = true,
			},
			[5] = {
				animationIDList = {	 },
				enabled = true,
			},
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Combo")
end


-- Check combopoints?
function Avion_Combo:UpdateEventRegistration(unLoad)
	local isEnabled = false
	for _, data in pairs(self.db.profile.ComboPoints) do
		if (data.enabled == true) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and isEnabled) then
		self:RegisterEvent("PLAYER_COMBO_POINTS")
	else
		if (self:IsEventRegistered("PLAYER_COMBO_POINTS")) then
			self:UnregisterEvent("PLAYER_COMBO_POINTS")
		end
	end
end


-- Build Optionstable
function Avion_Combo:BuildOptionTable()
	local data = self.db.profile	
	
	local cPointOptions = {}
	for cPoints, _ in pairs(data.ComboPoints) do
		cPointOptions[cPoints] = {
			order = 1,
			type = 'group',
			name = string.format(L["ComboPoint"], cPoints),
			desc = L["ComboPoint_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.ComboPoints[cPoints].enabled
					end,
					set = function(toggle)
						data.ComboPoints[cPoints].enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.ComboPoints[cPoints].animationIDList),
				},
			},
		}
	end
	
	-- AceOptionTable
	local aggroOptions = {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = cPointOptions	
	}
	
	return aggroOptions
end


-- Save and return
function Avion_Combo:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Combo:AnimationDeleted(animationID)
	for cPoints, data in pairs(self.db.profile.ComboPoints) do
		if (data.animationIDList[animationID]) then
			self.db.profile.ComboPoints[cPoints].animationIDList[animationID] = nil
		end
	end
end

	
-- Combo-Points updated
function Avion_Combo:PLAYER_COMBO_POINTS(unitID)
	local cPoints = GetComboPoints()
	local data = self.db.profile.ComboPoints
	
	-- Test if there is a trigger for this ...
	if (data[cPoints] and data[cPoints].enabled) then
		Avion_Core:TriggerAnimationsByID(string.format("%s%d", "ComboPoints", cPoints), data[cPoints].animationIDList)
	end
end
