-- If not found don't do anything
if (not GridStatus) then
	return
end


-- Create new Module
local Avion_Grid 	= Avion_Core:NewModule("Avion_Grid")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Grid")
local BC = AceLibrary("Babble-Class-2.2")


-- Module information, default options and name
Avion_Grid.name 	= "Avion_Grid"


-- Register SavedVariables
function Avion_Grid:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Grid", "profile", { 
		status = {
		},
		statusByName = {
		},
		statusByClass = {
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Grid")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Grid:UpdateEventRegistration(unLoad)
	local isEnabled = {
		gain = false,
		lost = false,
	}
	
	for _, data in pairs(self.db.profile.status) do
		if (data.enabled) then
			if (data.triggerOnLoose) then
				isEnabled.lost = true
			else
				isEnabled.gain = true
			end
		end
	end
	
	if (not (isEnabled.lost or isEnabled.gain)) then
		for _, data in pairs(self.db.profile.statusByName) do
			if (data.enabled) then
				if (data.triggerOnLoose) then
					isEnabled.lost = true
				else
					isEnabled.gain = true
				end
			end
		end
	end
	
	if (not (isEnabled.lost or isEnabled.gain)) then
		for _, data in pairs(self.db.profile.statusByClass) do
			if (data.enabled) then
				if (data.triggerOnLoose) then
					isEnabled.lost = true
				else
					isEnabled.gain = true
				end
			end
		end
	end

	if (not unLoad and isEnabled.gain) then
		self:RegisterEvent("Grid_StatusGained")
	else
		if (self:IsEventRegistered("Grid_StatusGained")) then
			self:UnregisterEvent("Grid_StatusGained")
		end
	end
	
	if (not unLoad and isEnabled.lost) then
		self:RegisterEvent("Grid_StatusLost")
	else
		if (self:IsEventRegistered("Grid_StatusLost")) then
			self:UnregisterEvent("Grid_StatusLost")
		end
	end
end


-- Convert Grid status to a more readable format
function Avion_Grid:ReadableStatus(statusID)
	for statusID_, _, descr_ in GridStatus:RegisteredStatusIterator() do
		if (statusID_ == statusID) then
			return descr_
		end
	end
	
	return "<Unknown status>"
end


-- Build Optionstable
function Avion_Grid:BuildOptionTable()
	local statusOpt = {}
	for statusID, data in pairs(self.db.profile.status) do
		statusOpt[statusID] = {
			type = 'group',
			name = self:ReadableStatus(statusID),
			desc = L["Status_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				triggerOnLoose = {
					order = 2,
					type = 'toggle',
					name = L["TriggerOnLoose"],
					desc = L["TriggerOnLoose_Desc"],
					get = function()
						return data.triggerOnLoose
					end,
					set = function(toggle)
						data.triggerOnLoose = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 3,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
			},
		}
	end
	
	local statusByNameOpt = {}
	local editStatusByNameOpt
	for statusID, data in pairs(self.db.profile.statusByName) do
		editStatusByNameOpt = {}
		for name, _ in pairs(data.names) do
			editStatusByNameOpt[name] =  {
				type = 'execute',
				name = name,
				desc = L["DelName_Desc"],
				func = function()
					data.names[name] = nil
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
				end,
			}
		end
		
		statusByNameOpt[statusID] = {
			type = 'group',
			name = self:ReadableStatus(statusID),
			desc = L["Status_Desc"],
			args = { 
				addName = {
					type = 'text',
					order = 1,
					name = L["AddName"],
					desc = L["AddName_Desc"],
					usage = L["AddName_Usage"],
					get = function()
						return ""
					end,
					set = function(input)
						if (self.db.profile.statusByName[statusID].names[input] or strlen(input) <= 0) then
							return 
						end
						
						-- Insert into DB
						self.db.profile.statusByName[statusID].names[input] = true
						
						-- Update option table
						Avion_Core:UpdateModuleOptionTable(self)
					end,
				},
				delName = {
					type = 'group',
					order = 2,
					name = L["DeleteName"],
					desc = L["DeleteName_Desc"],
					args = editStatusByNameOpt,
				},
				enabled = {
					order = 3,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				triggerOnLoose = {
					order = 4,
					type = 'toggle',
					name = L["TriggerOnLoose"],
					desc = L["TriggerOnLoose_Desc"],
					get = function()
						return data.triggerOnLoose
					end,
					set = function(toggle)
						data.triggerOnLoose = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 5,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
			},
		}
	end
	
	local statusByClassOpt = {}
	local editStatusByClassOpt, addClassOpt
	for statusID, data in pairs(self.db.profile.statusByClass) do
		addClassOpt = {}
		for _, class in pairs({"Druid", "Hunter", "Mage", "Paladin", "Priest", "Rogue", "Shaman", "Warlock", "Warrior"}) do
			if (not data.classes[class]) then
				addClassOpt[class] = {
					type = 'execute',
					name = BC[class],
					desc = L["AddClass_Desc"],
					func = function()
						if (data.classes[class]) then
							return 
						end
						
						-- Insert into DB
						data.classes[class] = true
						
						-- Update option table
						Avion_Core:UpdateModuleOptionTable(self)
					end,
				}
			end
		end
	
		editStatusByClassOpt = {}
		for class, _ in pairs(data.classes) do
			editStatusByClassOpt[class] =  {
				type = 'execute',
				name = BC[class],
				desc = L["DeleteClass_Desc"],
				func = function()
					data.classes[class] = nil
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
				end,
			}
		end
		
		statusByClassOpt[statusID] = {
			type = 'group',
			name = self:ReadableStatus(statusID),
			desc = L["Status_Desc"],
			args = { 
				addClass = {
					type = 'group',
					order = 1,
					name = L["AddClass"],
					name = L["AddClass"],
					desc = L["AddClass_Desc"],
					args = addClassOpt,
				},
				delClass = {
					type = 'group',
					order = 2,
					name = L["DeleteClass"],
					desc = L["DeleteClass_Desc"],
					args = editStatusByClassOpt,
				},
				enabled = {
					order = 3,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				triggerOnLoose = {
					order = 4,
					type = 'toggle',
					name = L["TriggerOnLoose"],
					desc = L["TriggerOnLoose_Desc"],
					get = function()
						return data.triggerOnLoose
					end,
					set = function(toggle)
						data.triggerOnLoose = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 5,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
			},
		}
	end
	
	-- Add new status
	local newStatusOpt = {}
	local delStatusOpt = {}
	for statusID, _, descr in GridStatus:RegisteredStatusIterator() do
		if (not (self.db.profile.status[statusID] and self.db.profile.statusByName[statusID] and self.db.profile.statusByClass[statusID])) then
			newStatusOpt[statusID] = {
				type = 'execute',
				name = descr,
				desc = L["Status_Desc"],
				func = function()
					-- Insert into DB
					self.db.profile.status[statusID] = {
						animationIDList = { },
						enabled = true,
						triggerOnLoose = false,
					}
					self.db.profile.statusByName[statusID] = {
						animationIDList = {},
						enabled = true,
						triggerOnLoose = false,
						names = { },
					}
					self.db.profile.statusByClass[statusID] = {
						animationIDList = { },
						enabled = true,
						triggerOnLoose = false,
						classes = {},
					}
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end,
			}
		else
			delStatusOpt[statusID] = {
				type = 'execute',
				name = descr,
				desc = L["DeleteEntry_Desc"],
				func = function()
					-- Delete from DB
					self.db.profile.status[statusID] = nil
					self.db.profile.statusByName[statusID] = nil
					self.db.profile.statusByClass[statusID] = nil
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end
			}
		end
	end

	-- AceOptionTable
	local aggroOptions = {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'group',
				order = 1,
				name = L["New"],
				desc = L["New_Desc"],
				args = newStatusOpt,
			},
			status = {
				type = 'group',
				order = 2,
				name = L["EditStatus"],
				desc = L["EditStatus_Desc"],
				args = statusOpt,
			},
			statusByName = {
				type = 'group',
				order = 3,
				name = L["EditStatusByName"],
				desc = L["EditStatusByName_Desc"],
				args = statusByNameOpt,
			},
			statusByClass = {
				type = 'group',
				order = 4,
				name = L["EditStatusByClass"],
				desc = L["EditStatusByClass_Desc"],
				args = statusByClassOpt,
			},
			delete = {
				order = 5,
				type = 'group',
				name = L["DeleteEntry"],
				desc = L["DeleteEntry_Desc"],
				args = delStatusOpt,
			},
		},
	}
	
	return aggroOptions
end


-- Save and return
function Avion_Grid:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Grid:AnimationDeleted(animationID)
	for _, data in pairs(self.db.profile.status) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	for _, data in pairs(self.db.profile.statusByName) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
	
	for _, data in pairs(self.db.profile.statusByClass) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
end


-- A status was gained
function Avion_Grid:Grid_StatusGained(name, status, priority, range, color, text, value, maxValue, texture) 
	-- Test if there is a trigger for this ...
	if (self.db.profile.statusByName[status] and self.db.profile.statusByName[status].enabled and not self.db.profile.statusByName[status].triggerOnLoose and self.db.profile.statusByName[status].names[name]) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusGained"..status, self.db.profile.statusByName[status].animationIDList)
	
	elseif (self.db.profile.statusByClass[status] and self.db.profile.statusByClass[status].enabled and not self.db.profile.statusByClass[status].triggerOnLoose and self.db.profile.statusByClass[status].classes[UnitClass(name)]) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusGained"..status, self.db.profile.statusByClass[status].animationIDList)
		
	elseif (self.db.profile.status[status] and self.db.profile.status[status].enabled and not self.db.profile.status[status].triggerOnLoose) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusGained"..status, self.db.profile.status[status].animationIDList)
	end
end


-- A status was lost
function Avion_Grid:Grid_StatusLost(name, status) 
	-- Test if there is a trigger for this ...
	if (self.db.profile.statusByName[status] and self.db.profile.statusByName[status].enabled and self.db.profile.statusByName[status].triggerOnLoose and self.db.profile.statusByName[status].names[name]) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusLost"..status, self.db.profile.statusByName[status].animationIDList)
	
	elseif (self.db.profile.statusByClass[status] and self.db.profile.statusByClass[status].enabled and self.db.profile.statusByClass[status].triggerOnLoose and self.db.profile.statusByClass[status].classes[UnitClass(name)]) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusLost"..status, self.db.profile.statusByClass[status].animationIDList)
		
	elseif (self.db.profile.status[status] and self.db.profile.status[status].enabled and self.db.profile.status[status].triggerOnLoose) then
		Avion_Core:TriggerAnimationsByID("Grid_StatusLost"..status, self.db.profile.status[status].animationIDList)
	end
end
