-- Create new Module
local Avion_Health 	= Avion_Core:NewModule("Avion_Health")


-- Translations
local L = AceLibrary("AceLocale-2.2"):new("Avion_Health")


-- Module information, default options and name
Avion_Health.name 	= "Avion_Health"


-- Register SavedVariables
function Avion_Health:RegisterModuleDB()
	Avion_Core:RegisterDefaults("Avion_Health", "profile", { 
		threshold = {
		},
	})
	
	self.db = Avion_Core:AcquireDBNamespace("Avion_Health")
	
	self.lastPercentage = UnitHealth("player")
end


-- (Un)register (un)needed events (aggro gain/loss)
function Avion_Health:UpdateEventRegistration(unLoad)
	local isEnabled
	
	isEnabled = false
	for _, data in pairs(self.db.profile.threshold) do
		if (data.enabled) then
			isEnabled = true
			break
		end
	end

	if (not unLoad and isEnabled) then
		self:RegisterEvent("UNIT_HEALTH")
	else
		if (self:IsEventRegistered("UNIT_HEALTH")) then
			self:UnregisterEvent("UNIT_HEALTH")
		end
	end
end


-- Build Optionstable
function Avion_Health:BuildOptionTable()
	local thresholdOpt = {}
	for index, data in pairs(self.db.profile.threshold) do
		thresholdOpt[index] = {
			type = 'group',
			name = L["Threshold"] .. index,
			desc = L["Threshold_Desc"],
			args = { 
				enabled = {
					order = 1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled_Desc"],
					get = function()
						return data.enabled
					end,
					set = function(toggle)
						data.enabled = toggle
						
						self:UpdateEventRegistration(false)
					end,
				},
				list = {
					order = 2,
					type = 'group',
					name = L["AnimationList"],
					desc = L["AnimationList_Desc"],
					args = Avion_Core:BuildAnimationList(data.animationIDList),
				},
				delete = {
					order = 3,
					type = 'group',
					name = L["DeleteEntry"],
					desc = L["DeleteEntry_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteEntry"],
							desc = L["DeleteEntry_Desc"],
							func = function()
								-- Delete from DB
								self.db.profile.threshold[index] = nil
								
								-- Update option table
								Avion_Core:UpdateModuleOptionTable(self)
								
								-- Update events
								self:UpdateEventRegistration(false)
							end
						},
					},
				},
			},
		}
	end

	-- AceOptionTable
	return {
		type = 'group',
		order = 2,
		name = L["ModuleName"],
		desc = L["ModuleName_Desc"],
		args = {
			new = {
				type = 'text',
				order = 1,
				name = L["New"],
				desc = L["New_Desc"],
				usage = L["New_Usage"],
				get = function()
					return ""
				end,
				set = function(input)
					local no = tonumber(input)
					
					if (self.db.profile.threshold[no] or strlen(input) == 0) then
						return
					end
					
					-- Insert into DB
					self.db.profile.threshold[no] = {
						animationIDList = { },
						enabled = true,
					}
					
					-- Update option table
					Avion_Core:UpdateModuleOptionTable(self)
					
					-- Update events
					self:UpdateEventRegistration(false)
				end,
			},
			edit = {
				type = 'group',
				order = 2,
				name = L["Edit"],
				desc = L["Edit_Desc"],
				args = thresholdOpt,
			},
		},
	}
end


-- Save and return
function Avion_Health:UpdateOptionTable()
	self.onMenuRequest = self:BuildOptionTable()
	
	return self.onMenuRequest
end


-- Animation was deleted, make sure its not used anymore
function Avion_Health:AnimationDeleted(animationID)
	for _, data in pairs(self.db.profile.threshold) do
		if (data.animationIDList[animationID]) then
			data.animationIDList[animationID] = nil
		end
	end
end


-- Fire or update low health event
function Avion_Health:UNIT_HEALTH(unitID)	
	-- But only for player
	if (unitID ~= "player") then
		return
	end
	
	-- Save current health
	local currentPercentage = UnitHealth("player") / UnitHealthMax("player") * 100
	
	-- check if health is growing or shrinking
	if (currentPercentage < self.lastPercentage) then
		-- Test each treshhold
		for threshold, data in pairs(self.db.profile.threshold) do
			-- Check if health is inside interval [newHealth - oldHealth)
			if (data.enabled and currentPercentage <= threshold and threshold < self.lastPercentage) then
				Avion_Core:TriggerAnimationsByID("HealthThreshold", data.animationIDList)
			end
		end
	end
	
	-- Update current/last health
	self.lastPercentage = currentPercentage
end
