﻿--[[
Name: AvionAddin-1.0
Revision: $Revision: 1 $
Author(s): Sirow (EU-Gul'dan)
Documentation: http://www.wowace.com/wiki/AvionAddin-1.0
SVN: http://svn.wowace.com/wowace/trunk/LibAvion/
Description: Adds options and many more to Avion-1.0.
Dependencies: LibAvion-1.0, AceDB-2.0, AceEvent-2.0, LibSharedMedia-2.0
]]


-------------------------------------------------------------------------------
-- Preamble
-------------------------------------------------------------------------------
-- Version control
local MAJOR_VERSION = "AvionAddin-1.0"
local MINOR_VERSION = "$Revision: 2 $"

-- Take a loot at main-dependency
if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

-- Make sure sup-dependencies are also there
if not AceLibrary:HasInstance("Avion-1.0") then error(MAJOR_VERSION .. " requires Avion-1.0.") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0.") end
if not AceLibrary:HasInstance("AceDB-2.0") then error(MAJOR_VERSION .. " requires AceDB-2.0.") end
if not AceLibrary:HasInstance("LibSharedMedia-2.0") then error(MAJOR_VERSION .. " requires LibSharedMedia-2.0.") end

-- Library holder
local AvionAddin 	= {}
local AceEvent 		= AceLibrary("AceEvent-2.0")
local AceDB 		= AceLibrary("AceDB-2.0")

-- Embed them into this lib
AceEvent:embed(AvionAddin)
AceDB:embed(AvionAddin)


-------------------------------------------------------------------------------
-- Translation
-------------------------------------------------------------------------------
-- Translations
local L = AceLibrary("AceLocale-2.2"):new("AvionAddin-1.0")


-- English translation
L:RegisterTranslations("enUS", function() return {
	["OpenMenu"] = "OpenMenu",
	["OpenMenu_Desc"] = "Open the options menu",
	
	["StartValues"] = "Start values",
	["StartValues_Desc"] = "Change values for showing frame",
	["AnimationValues"] = "Animate values",
	["AnimationValues_Desc"] = "Change values for animating frame",
	["AnimationOption_Desc"] = "Change settings regarding this animation",
	
	["Enabled"] = "Enabled",
	["Disabled"] = "Disabled",
	["IsAnimated"] = "Animate",
	["IsAnimated_Desc"] = "What should be animated",
	["AnimatedFalse"] = "Not animated",
	["AnimatedTrue"] = "Is animated",
	["AddRandom"] = "Random",
	["AddRandom_Desc"] = "Add some randomness to animation",
	["AnimationFunc"] = "Function",
	["AnimationFunc_Desc"] = "Which function to use for animation",
	
	["Name"] = "Name",
	["Name_Desc"] = "Change name of the animation",
	["Name_Usage"] = "<name>",
	["Color"] = "Color",
	["Color_Desc"] = "Change color",
	["ColorAlpha"] = "Color/Alpha",
	["ColorAlpha_Desc"] = "Change color and alpha",
	["PositionX"] = "X-Position",
	["PositionX_Desc"] = "Change X-Position",
	["PositionY"] = "Y-Position",
	["PositionY_Desc"] = "Change Y-Position",
	["Width"] = "Width",
	["Width_Desc"] = "Change width",
	["Height"] = "Height",
	["Height_Desc"] = "Change height",
	["Rotation"] = "Rotation",
	["Rotation_Desc"] = "Change rotation",
	["Scale"] = "Scale",
	["Scale_Desc"] = "Change scale",
	["Position"] = "Position",
	["Position_Desc"] = "Change position",
	["Alpha"] = "Alpha",
	["Alpha_Desc"] = "Change alpha",
	["AnimateAndFade"] = "Animate and fade",
	["AnimateAndFade_Desc"] = "Stop animation when fading or continue",
	["Durations"] = "Durations",
	["Durations_Desc"] = "Duration of animation",
	["AnimationTime"] = "Animate",
	["AnimationTime_Desc"] = "Main animation time",
	["FadeTime"] = "Fade",
	["FadeTime_Desc"] = "Duration for fading",
	["WaitTime"] = "Wait",
	["WaitTime_Desc"] = "Time to wait until showing animation",
	["Texture"] = "Texture",
	["Texture_Desc"] = "Change animation texture",
	["Sound"] = "Sound",
	["Sound_Desc"] = "Sound to play when triggered",
	["RunAnimation"] = "Trigger",
	["RunAnimation_Desc"] = "Trigger this animation",
	["ShowWhileWait"] = "Show while waiting",
	["ShowWhileWait_Desc"] = "Show the frame while waiting timer runs",
	["AvionSettings"] = "Avion settings",
	["AvionSettings_Desc"] = "Change Avion settings",
	["AnimationSettings"] = "Edit animation",
	["AnimationSettings_Desc"] = "Change settings of a certain animation",
	["Hide"] = "Hide",
	["Show"] = "Show",
	["ShowHide_Desc"] = "Show testframe for this animation",
	["TestSettings"] = "Testing",
	["TestSettings_Desc"] = "Test this animation",
	["Timer"] = "Timer",
	["Timer_Desc"] = "Cahnge testframe timer",
	["NewAnimation"] = "New animation",
	["NewAnimation_Desc"] = "Create a new animation",
	["NewAnimationDefault"] = "Default",
	["NewAnimationDefault_Desc"] = "Create from default settings",
	["NewAnimationCopy"] = "Copy",
	["NewAnimationCopy_Desc"] = "Copy from available animation",
	["NewAnimation_Usage"] = "<name>",
	["DeleteAnimation"] = "Delete",
	["DeleteAnimation_Desc"] = "Delete this animation",
	["Animation_Desc"] = "Should this animation be triggered?",
} end)
-- Chinese translation [CWDG]jiyun
L:RegisterTranslations("zhCN", function() return {
	["OpenMenu"] = "\230\137\147\229\188\128\232\143\156\229\141\149",
	["OpenMenu_Desc"] = "\230\137\147\229\188\128\233\128\137\233\161\185\232\143\156\229\141\149",
	
	["StartValues"] = "\232\181\183\229\167\139\229\128\188",
	["StartValues_Desc"] = "\230\155\180\230\148\185\230\161\134\230\158\182\230\152\190\231\164\186\231\154\132\230\149\176\229\128\188",
	["AnimationValues"] = "\229\138\168\230\128\129\230\149\136\230\158\156\229\128\188",
	["AnimationValues_Desc"] = "\230\155\180\230\148\185\230\161\134\230\158\182\229\138\168\230\128\129\230\149\136\230\158\156\231\154\132\230\149\176\229\128\188",
	["AnimationOption_Desc"] = "\230\155\180\230\148\185\232\191\153\228\184\170\229\138\168\230\128\129\230\149\136\230\158\156\232\174\190\231\189\174",
	
	["Enabled"] = "\229\188\128\229\144\175",
	["Disabled"] = "\229\133\179\233\151\173",
	["IsAnimated"] = "\229\138\168\230\128\129",
	["IsAnimated_Desc"] = "\230\152\175\229\144\166\232\174\190\231\189\174\228\184\186\229\138\168\230\128\129\230\149\136\230\158\156",
	["AnimatedFalse"] = "\228\184\141\228\189\191\231\148\168",
	["AnimatedTrue"] = "\228\189\191\231\148\168",
	["AddRandom"] = "\233\154\143\230\156\186",
	["AddRandom_Desc"] = "\229\138\160\230\183\187\228\184\128\228\186\155\233\154\143\230\156\186\231\154\132\229\138\168\230\128\129\230\149\136\230\158\156",
	["AnimationFunc"] = "\229\138\159\232\131\189",
	["AnimationFunc_Desc"] = "\228\189\191\231\148\168\229\138\168\230\128\129\230\149\136\230\158\156\231\154\132\233\130\163\228\186\155\229\138\159\232\131\189",
	
	["Name"] = "\229\144\141\231\167\176",
	["Name_Desc"] = "\230\155\180\230\148\185\229\138\168\230\128\129\230\149\136\230\158\156\231\154\132\229\144\141\231\167\176",
	["Name_Usage"] = "<\229\144\141\231\167\176>",
	["Color"] = "\233\162\156\232\137\178",
	["Color_Desc"] = "\230\155\180\230\148\185\233\162\156\232\137\178",
	["ColorAlpha"] = "\233\162\156\232\137\178/\233\128\143\230\152\142",
	["ColorAlpha_Desc"] = "\230\155\180\230\148\185\233\162\156\232\137\178\228\184\142\233\128\143\230\152\142\229\186\166",
	["PositionX"] = "X\232\189\180",
	["PositionX_Desc"] = "\230\155\180\230\148\185X\232\189\180",
	["PositionY"] = "Y\232\189\180",
	["PositionY_Desc"] = "\230\155\180\230\148\185Y\232\189\180",
	["Width"] = "\229\174\189\229\186\166",
	["Width_Desc"] = "\230\155\180\230\148\185\229\174\189\229\186\166",
	["Height"] = "\233\171\152\229\186\166",
	["Height_Desc"] = "\230\155\180\230\148\185\233\171\152\229\186\166",
	["Rotation"] = "\230\151\139\232\189\172",
	["Rotation_Desc"] = "\230\155\180\230\148\185\230\151\139\232\189\172\232\167\146\229\186\166",
	["Scale"] = "\230\175\148\228\190\139",
	["Scale_Desc"] = "\230\155\180\230\148\185\230\175\148\228\190\139",
	["Position"] = "\228\189\141\231\189\174",
	["Position_Desc"] = "\230\155\180\230\148\185\228\189\141\231\189\174",
	["Alpha"] = "\233\128\143\230\152\142\229\186\166",
	["Alpha_Desc"] = "\230\155\180\230\148\185\233\128\143\230\152\142\229\186\166",
	["AnimateAndFade"] = "\229\138\168\230\128\129\230\149\136\230\158\156\228\184\142\230\184\144\233\154\144",
	["AnimateAndFade_Desc"] = "\229\129\156\230\173\162\229\138\168\231\148\187\230\151\182\239\188\140\230\184\144\233\154\144\230\136\150\231\187\167\231\187\173",
	["Durations"] = "\230\140\129\231\187\173\230\151\182\233\151\180",
	["Durations_Desc"] = "\229\138\168\230\128\129\230\149\136\230\158\156\230\140\129\231\187\173\230\151\182\233\151\180",
	["AnimationTime"] = "\229\138\168\231\148\187",
	["AnimationTime_Desc"] = "\228\184\187\229\138\168\231\148\187\230\151\182\233\151\180",
	["FadeTime"] = "\230\184\144\233\154\144",
	["FadeTime_Desc"] = "\230\184\144\233\154\144\230\140\129\231\187\173\230\151\182\233\151\180",
	["WaitTime"] = "\231\173\137\229\190\133",
	["WaitTime_Desc"] = "\230\152\190\231\164\186\229\138\168\230\128\129\230\149\136\230\158\156\231\154\132\230\151\182\233\151\180",
	["Texture"] = "\230\157\144\232\180\168",
	["Texture_Desc"] = "\230\155\180\230\148\185\229\138\168\230\128\129\230\149\136\230\158\156\231\154\132\230\157\144\232\180\168",
	["Sound"] = "\229\163\176\230\149\136",
	["Sound_Desc"] = "\230\149\136\230\158\156\229\135\186\229\143\145\230\146\173\230\148\190\229\163\176\230\149\136",
	["RunAnimation"] = "\232\167\166\229\143\145",
	["RunAnimation_Desc"] = "\229\138\168\230\128\129\230\149\136\230\158\156\232\167\166\229\143\145\230\157\161\228\187\182",
	["ShowWhileWait"] = "\230\152\190\231\164\186\232\191\144\232\161\140\230\149\136\230\158\156",
	["ShowWhileWait_Desc"] = "\231\173\137\229\190\133\232\174\161\230\151\182\230\157\161\232\191\144\232\161\140\230\151\182\230\152\190\231\164\186\230\161\134\228\189\147",
	["AvionSettings"] = "Avion \232\174\190\231\189\174",
	["AvionSettings_Desc"] = "\230\155\180\230\148\185 Avion \232\174\190\231\189\174",
	["AnimationSettings"] = "\231\188\150\232\190\145\229\138\168\230\128\129\230\149\136\230\158\156",
	["AnimationSettings_Desc"] = "\230\155\180\230\148\185\230\168\161\229\134\153\229\138\168\230\128\129\230\149\136\230\158\156\232\174\190\231\189\174",
	["Hide"] = "\233\154\144\232\151\143",
	["Show"] = "\230\152\190\231\164\186",
	["ShowHide_Desc"] = "\230\152\190\231\164\186\229\138\168\230\128\129\230\181\139\232\175\149\230\149\136\230\158\156",
	["TestSettings"] = "\230\181\139\232\175\149\228\184\173",
	["TestSettings_Desc"] = "\230\181\139\232\175\149\230\173\164\230\149\136\230\158\156",
	["Timer"] = "\232\174\161\230\151\182\229\153\168",
	["Timer_Desc"] = "\230\155\180\230\148\185\230\181\139\232\175\149\232\174\161\230\151\182\229\153\168",
	["NewAnimation"] = "\230\150\176\229\187\186\229\138\168\230\128\129\230\149\136\230\158\156",
	["NewAnimation_Desc"] = "\229\136\155\233\128\160\230\150\176\231\154\132\229\138\168\230\128\129\230\149\136\230\158\156",
	["NewAnimationDefault"] = "\233\187\152\232\174\164",
	["NewAnimationDefault_Desc"] = "\229\187\186\233\135\140\233\187\152\232\174\164\232\174\190\231\189\174",
	["NewAnimationCopy"] = "\229\164\141\229\136\182",
	["NewAnimationCopy_Desc"] = "\229\164\141\229\136\182\229\143\175\231\148\168\230\149\136\230\158\156",
	["NewAnimation_Usage"] = "<\229\144\141\231\167\176>",
	["DeleteAnimation"] = "\229\136\160\233\153\164",
	["DeleteAnimation_Desc"] = "\229\136\160\233\153\164\230\173\164\230\149\136\230\158\156",
	["Animation_Desc"] = "\229\186\148\231\148\168\230\173\164\229\138\168\230\128\129\230\149\136\230\158\156\232\167\166\229\143\145\239\188\159",
} end)
-- 우리말
L:RegisterTranslations("koKR", function() return {
	["OpenMenu"] = "메뉴 열기",
	["OpenMenu_Desc"] = "설정창을 엽니다",
	
	["StartValues"] = "시작값",
	["StartValues_Desc"] = "시작시 설정값을 변경합니다",
	["AnimationValues"] = "변동값",
	["AnimationValues_Desc"] = "애니메이션 변동시 설정값을 변경합니다",
	["AnimationOption_Desc"] = "이 애니메이션에 따른 설정을 변경합니다",
	
	["Enabled"] = "사용",
	["Disabled"] = "사용안함",
	["IsAnimated"] = "움직임",
	["IsAnimated_Desc"] = "어떤 움직임으로 할것인지 선택합니다",
	["AnimatedFalse"] = "움직이지 않음",
	["AnimatedTrue"] = "움직이는중",
	["AddRandom"] = "무작위",
	["AddRandom_Desc"] = "무작위 움직임 효과를 추가합니다",
	["AnimationFunc"] = "기능",
	["AnimationFunc_Desc"] = "움직임 효과에 사용할 기능을 선택합니다",
	
	["Name"] = "이름",
	["Name_Desc"] = "애니메이션의 이름을 변경합니다",
	["Name_Usage"] = "<이름>",
	["Color"] = "색상",
	["Color_Desc"] = "색상을 변경합니다",
	["ColorAlpha"] = "색상/투명도",
	["ColorAlpha_Desc"] = "색상과 투명도를 변경합니다",
	["PositionX"] = "가로 위치",
	["PositionX_Desc"] = "가로 위치를 변경합니다",
	["PositionY"] = "세로 위치",
	["PositionY_Desc"] = "세로 위치를 변경합니다",
	["Width"] = "너비",
	["Width_Desc"] = "너비를 변경합니다",
	["Height"] = "높이",
	["Height_Desc"] = "높이를 변경합니다",
	["Rotation"] = "회전",
	["Rotation_Desc"] = "회전 각도를 변경합니다",
	["Scale"] = "크기",
	["Scale_Desc"] = "크기를 변경합니다",
	["Position"] = "위치",
	["Position_Desc"] = "위치를 변경합니다",
	["Alpha"] = "투명도",
	["Alpha_Desc"] = "투명도를 변경합니다",
	["AnimateAndFade"] = "움직임 후에 사라짐 효과",
	["AnimateAndFade_Desc"] = "움직임이 종료 된 후에 사라짐 효과를 적용할지 아닐지를 설정합니다",
	["Durations"] = "지속시간",
	["Durations_Desc"] = "애니메이션의 지속시간을 설정합니다",
	["AnimationTime"] = "움직임",
	["AnimationTime_Desc"] = "실제 움직임 시간을 설정합니다",
	["FadeTime"] = "사라짐",
	["FadeTime_Desc"] = "사라짐 효과의 지속 시간을 설정합니다",
	["WaitTime"] = "대기",
	["WaitTime_Desc"] = "애니메이션 표시시 까지의 대기 시간을 설정합니다",
	["Texture"] = "텍스쳐",
	["Texture_Desc"] = "애니메이션 텍스쳐를 변경합니다",
	["Sound"] = "소리",
	["Sound_Desc"] = "반응시 재생할 소리를 변경합니다",
	["RunAnimation"] = "반응",
	["RunAnimation_Desc"] = "이 애니메이션에 반응합니다",
	["ShowWhileWait"] = "대기중에 표시",
	["ShowWhileWait_Desc"] = "대기 시간 중에도 애니메이션을 표시합니다",
	["AvionSettings"] = "Avion 설정",
	["AvionSettings_Desc"] = "Avion의 설정을 변경합니다",
	["AnimationSettings"] = "애니메이션 편집",
	["AnimationSettings_Desc"] = "특정 애니메이션의 설정을 변경합니다",
	["Hide"] = "숨김",
	["Show"] = "표시",
	["ShowHide_Desc"] = "이 애니메이션을 시험적으로 표시합니다",
	["TestSettings"] = "시험",
	["TestSettings_Desc"] = "이 애니메이션을 시험 작동합니다",
	["Timer"] = "타이머",
	["Timer_Desc"] = "테스트 표시의 타이머를 변경합니다",
	["NewAnimation"] = "신규 애니메이션",
	["NewAnimation_Desc"] = "새로운 애이메이션을 생성합니다",
	["NewAnimationDefault"] = "기본값",
	["NewAnimationDefault_Desc"] = "기본값으로 신규 생성합니다",
	["NewAnimationCopy"] = "복사",
	["NewAnimationCopy_Desc"] = "다른 애니메이션의 설정을 복사합니다",
	["NewAnimation_Usage"] = "<이름>",
	["DeleteAnimation"] = "삭제",
	["DeleteAnimation_Desc"] = "이 애니메이션 효과를 삭제합니다",
	["Animation_Desc"] = "정말로 이 애니메이션 효과를 적용합니까?",
} end)

-------------------------------------------------------------------------------
-- Library - Locals
-------------------------------------------------------------------------------
local Media
local Avion


-------------------------------------------------------------------------------
-- Library - Core
-------------------------------------------------------------------------------
-- Default options
function AvionAddin:BuildDefaultOptions()
	return {
		order = 1,
		type = 'group',
		name = L["AvionSettings"],
		desc = L["AvionSettings_Desc"],
		args = {
			individual = {
				order = 1,
				type = 'group',
				name = L["AnimationSettings"],
				desc = L["AnimationSettings_Desc"],
				args = { },
			},
			new = {
				order = 2,
				type = 'group',
				name = L["NewAnimation"],
				desc = L["NewAnimation_Desc"],
				args = {
					default = {
						order = 1,
						type = 'text',
						name = L["NewAnimationDefault"],
						desc = L["NewAnimationDefault_Desc"],
						usage = L["NewAnimation_Usage"],
						get = function()
							return ""
						end,
						set = function(new)
							self:NewAnimation(nil, new)
						end,
						validate = function(new)
							return (strlen(new) > 0)
						end,
					},
					copy = {
						order = 1,
						type = 'group',
						name = L["NewAnimationCopy"],
						desc = L["NewAnimationCopy_Desc"],
						args = { },
					},
				},
			},
		},
	}
end


-- Build option-table for given animation
function AvionAddin:BuildAnimationOption(animationID)
	if (self.vars.animationList[animationID]) then
		return {
			order = animationID,
			type = 'group',
			name = string.format("%s%d%s", "<", animationID, "> " .. self.vars.animationList[animationID].name),
			desc = L["AnimationOption_Desc"],
			args = {
				name = {
					order = 1,
					type = 'text',
					name = L["Name"],
					desc = L["Name_Desc"],
					usage = L["Name_Usage"],
					get = function()
						return self.vars.animationList[animationID].name
					end,
					set = function(newName)
						self.vars.animationList[animationID].name = newName
						
						self:UpdateOptionTable()
					end,
				},
				startValues = {
					order = 2,
					type = 'group',
					name = L["StartValues"],
					desc = L["StartValues_Desc"],
					args = { 
						color = {
							order = 1,
							type = 'color',
							name = L["ColorAlpha"],
							desc = L["ColorAlpha_Desc"],
							get = function()
								return self.vars.animationList[animationID].color.r, self.vars.animationList[animationID].color.g, self.vars.animationList[animationID].color.b, self.vars.animationList[animationID].alpha
							end,
							set = function(r, g, b, a)
								self.vars.animationList[animationID].color.r = r
								self.vars.animationList[animationID].color.g = g
								self.vars.animationList[animationID].color.b = b
								self.vars.animationList[animationID].alpha = a
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							hasAlpha = true
						},
						positionX = {
							order = 2,
							type = 'range',
							name = L["PositionX"],
							desc = L["PositionX_Desc"],
							get = function()
								return self.vars.animationList[animationID].position.x
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].position.x = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						positionY = {
							order = 3,
							type = 'range',
							name = L["PositionY"],
							desc = L["PositionY_Desc"],
							get = function()
								return self.vars.animationList[animationID].position.y
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].position.y = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						width = {
							order = 4,
							type = 'range',
							name = L["Width"],
							desc = L["Width_Desc"],
							get = function()
								return self.vars.animationList[animationID].size.width
							end,
							set = function(newWidth)
								self.vars.animationList[animationID].size.width = newWidth
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale()),
							step = 1,
						},
						height = {
							order = 5,
							type = 'range',
							name = L["Height"],
							desc = L["Height_Desc"],
							get = function()
								return self.vars.animationList[animationID].size.height
							end,
							set = function(newHeight)
								self.vars.animationList[animationID].size.height = newHeight
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max =  math.floor(GetScreenHeight() * 1 / UIParent:GetScale()),
							step = 1,
						},
						rotation = {
							order = 6,
							type = 'range',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].rotation / math.pi * 180
							end,
							set = function(newRotation)
								self.vars.animationList[animationID].rotation = newRotation * math.pi / 180
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 359,
							step = 1,
						},
					},
				},
				animation = {
					order = 3,
					type = 'group',
					name = L["AnimationValues"],
					desc = L["AnimationValues_Desc"],
					args = { 
						color = {
							order = 1,
							type = 'color',
							name = L["ColorAlpha"],
							desc = L["ColorAlpha_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.color.r, self.vars.animationList[animationID].animation.color.g, self.vars.animationList[animationID].animation.color.b
							end,
							set = function(r, g, b, a)
								self.vars.animationList[animationID].animation.color.r = r
								self.vars.animationList[animationID].animation.color.g = g
								self.vars.animationList[animationID].animation.color.b = b
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						positionX = {
							order = 2,
							type = 'range',
							name = L["PositionX"],
							desc = L["PositionX_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.position.x
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].animation.position.x = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						positionY = {
							order = 3,
							type = 'range',
							name = L["PositionY"],
							desc = L["PositionY_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.position.y
							end,
							set = function(newPosition)
								self.vars.animationList[animationID].animation.position.y = newPosition
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
							step = 1,
						},
						rotation = {
							order = 4,
							type = 'range',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.rotation / math.pi * 180
							end,
							set = function(newRotation)
								self.vars.animationList[animationID].animation.rotation = newRotation * math.pi / 180
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 359,
							step = 1,
						},
						scale = {
							order = 5,
							type = 'range',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								return self.vars.animationList[animationID].animation.scale
							end,
							set = function(newScale)
								self.vars.animationList[animationID].animation.scale = newScale
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0.1,
							max = 10,
							step = 0.05,
						},
					},
				},
				animationFunc = {
					order = 4,
					type = 'group',
					name = L["AnimationFunc"],
					desc = L["AnimationFunc_Desc"],
					args = { 
						alpha = {
							order = 1,
							type = 'text',
							name = L["Alpha"],
							desc = L["Alpha_Desc"],
							validate = self:FunctionList(),
							get = function()
								return self.vars.animationList[animationID].animationFunc.alpha
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.alpha = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						},
						color = {
							order = 2,
							type = 'text',
							name = L["Color"],
							desc = L["Color_Desc"],
							validate = self:FunctionList(),
							get = function()
								return self.vars.animationList[animationID].animationFunc.color
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.color = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						position = {
							order = 3,
							type = 'text',
							name = L["Position"],
							desc = L["Position_Desc"],
							validate = self:FunctionList(),
							get = function()
								return self.vars.animationList[animationID].animationFunc.position
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.position = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						scale = {
							order = 4,
							type = 'text',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							validate = self:FunctionList(),
							get = function()
								return self.vars.animationList[animationID].animationFunc.scale
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.scale = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
						rotation = {
							order = 5,
							type = 'text',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							validate = self:FunctionList(),
							get = function()
								return self.vars.animationList[animationID].animationFunc.rotation
							end,
							set = function(new)
								self.vars.animationList[animationID].animationFunc.rotation = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
						}, 
					},
				},
				addRandom = {
					order = 5,
					type = 'group',
					name = L["AddRandom"],
					desc = L["AddRandom_Desc"],
					args = {
						color = {
							order = 1,
							type = 'range',
							name = L["Color"],
							desc = L["Color_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.color) then
									return self.vars.animationList[animationID].addRandom.color
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.color = nil
								else
									self.vars.animationList[animationID].addRandom.color = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						position = {
							order = 2,
							type = 'range',
							name = L["Position"],
							desc = L["Position_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.position) then
									return self.vars.animationList[animationID].addRandom.position
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.position = nil
								else
									self.vars.animationList[animationID].addRandom.position = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						scale = {
							order = 3,
							type = 'range',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.scale) then
									return self.vars.animationList[animationID].addRandom.scale
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.scale = nil
								else
									self.vars.animationList[animationID].addRandom.scale = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
						rotation = {
							order = 4,
							type = 'range',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								if (self.vars.animationList[animationID].addRandom.rotation) then
									return self.vars.animationList[animationID].addRandom.rotation
								else
									return 0
								end
							end,
							set = function(newRandom)
								if (newRandom == 0) then
									self.vars.animationList[animationID].addRandom.rotation = nil
								else
									self.vars.animationList[animationID].addRandom.rotation = newRandom
								end
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							min = 0,
							max = 250,
							step = 5,
						},
					},
				},
				isAnimated = {
					order = 6,
					type = 'group',
					name = L["IsAnimated"],
					desc = L["IsAnimated_Desc"],
					args = {
						color = {
							order = 1,
							type = 'toggle',
							name = L["Color"],
							desc = L["Color_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.color
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.color = toggle
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							map = { [false] = L["AnimatedFalse"], [true] = L["AnimatedTrue"] }
						},
						position = {
							order = 2,
							type = 'toggle',
							name = L["Position"],
							desc = L["Position_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.position
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.position = toggle
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							map = { [false] = L["AnimatedFalse"], [true] = L["AnimatedTrue"] }
						},
						scale = {
							order = 3,
							type = 'toggle',
							name = L["Scale"],
							desc = L["Scale_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.scale
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.scale = toggle
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							map = { [false] = L["AnimatedFalse"], [true] = L["AnimatedTrue"] }
						},
						rotation = {
							order = 4,
							type = 'toggle',
							name = L["Rotation"],
							desc = L["Rotation_Desc"],
							get = function()
								return self.vars.animationList[animationID].isAnimated.rotation
							end,
							set = function(toggle)
								self.vars.animationList[animationID].isAnimated.rotation = toggle
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
							end,
							map = { [false] = L["AnimatedFalse"], [true] = L["AnimatedTrue"] }
						},
					},
				},
				timing = {
					order = 7,
					type = 'group',
					name = L["Durations"],
					desc = L["Durations_Desc"],
					args = { 
						waitTime = {
							order = 1,
							type = 'range',
							name = L["WaitTime"],
							desc = L["WaitTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].waitTime
							end,
							set = function(new)
								self.vars.animationList[animationID].waitTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:UpdateOptionTable()
							end,
							min = 0,
							max = 10,
							step = self.vars.updateInterval,
						},
						animationTime = {
							order = 2,
							type = 'range',
							name = L["AnimationTime"],
							desc = L["AnimationTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].animationTime
							end,
							set = function(new)
								self.vars.animationList[animationID].animationTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:UpdateOptionTable()
							end,
							min = 0,
							max = 10,
							step = Avion.vars.updateInterval,
						},
						fadeTime = {
							order = 3,
							type = 'range',
							name = L["FadeTime"],
							desc = L["FadeTime_Desc"],
							get = function()
								return self.vars.animationList[animationID].fadeTime
							end,
							set = function(new)
								self.vars.animationList[animationID].fadeTime = new
								
								self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
								
								self:UpdateOptionTable()
							end,
							min = 0,
							max = 10,
							step = Avion.vars.updateInterval,
						},
					},
				},
				texture = {
					order = 8,
					type = 'text',
					name = L["Texture"],
					desc = L["Texture_Desc"],
					validate = Media:List("aura"),
					get = function()
						return self.vars.animationList[animationID].textureFile
					end,
					set = function(new)
						self.vars.animationList[animationID].textureFile = new
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
				},
				sound = {
					order = 9,
					type = 'text',
					name = L["Sound"],
					desc = L["Sound_Desc"],
					validate = Media:List("sound"),
					get = function()
						if (self.vars.animationList[animationID].soundFile == nil) then
							return "None"
						else
							return self.vars.animationList[animationID].soundFile
						end
					end,
					set = function(new)
						if (new == "None") then
							self.vars.animationList[animationID].soundFile = nil
						else
							self.vars.animationList[animationID].soundFile = new
							
							PlaySoundFile(Media:Fetch("sound", new))
						end
					end,
				},
				animateAndFade = {
					order = 10,
					type = 'toggle',
					name = L["AnimateAndFade"],
					desc = L["AnimateAndFade_Desc"],
					get = function()
						return self.vars.animationList[animationID].animateAndFade
					end,
					set = function(toggle)
						self.vars.animationList[animationID].animateAndFade = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
					map = { [false] = L["Enabled"], [true] = L["Disabled"] }
				},
				showWhileWait = {
					order = 10,
					type = 'toggle',
					name = L["ShowWhileWait"],
					desc = L["ShowWhileWait_Desc"],
					get = function()
						return self.vars.animationList[animationID].showWhileWait
					end,
					set = function(toggle)
						self.vars.animationList[animationID].showWhileWait = toggle
						
						self:SetTestFrameStatus(animationID, self:GetTestFrameStatus(animationID))
					end,
					map = { [false] = L["Enabled"], [true] = L["Disabled"] }
				},
				testing = {
					order = 11,
					type = 'group',
					name = L["TestSettings"],
					desc = L["TestSettings_Desc"],
					args = {
						runAnimation = {
							order = 1,
							type = 'execute',
							name = L["RunAnimation"],
							desc = L["RunAnimation_Desc"],
							func = function()
								self:StopAnimation(string.format("%s%d", "Avion_RunAnimation_", animationID))
								self:TriggerAnimationByID(string.format("%s%d", "Avion_RunAnimation_", animationID), animationID)
							end
						},
						showHide = {
							order = 2,
							type = 'toggle',
							name = L["Show"],
							desc = L["ShowHide_Desc"],
							get = function()
								return self:IsTestFrameShown(animationID)
							end,
							set = function(toggle)
								if (toggle == true) then
									self:ShowTestFrame(animationID)
								else
									self:HideTestFrame(animationID)
								end
							end,
							map = { [false] = L["Hide"], [true] = L["Show"] }
						},
						elapsedTimes = {
							order = 3,
							type = 'range',
							name = L["Timer"],
							desc = L["Timer_Desc"],
							get = function()
								return self:GetTestFrameStatus(animationID)
							end,
							set = function(new)
								self:SetTestFrameStatus(animationID, new)
							end,
							min = 0,
							max = self.vars.animationList[animationID].waitTime + self.vars.animationList[animationID].animationTime + self.vars.animationList[animationID].fadeTime,
							step = Avion.vars.updateInterval,
						},
					},
				},
				delete = {
					order = 12,
					type = 'group',
					name = L["DeleteAnimation"],
					desc = L["DeleteAnimation_Desc"],
					args = {
						delete = {
							order = 1,
							type = 'execute',
							name = L["DeleteAnimation"],
							desc = L["DeleteAnimation_Desc"],
							func = function()
								self:DelAnimation(animationID)
							end
						},
					},
				},
			},
		}
	else
		return nil
	end
end


-- Create new animation as copy from existing one
function AvionAddin:BuildCopyOption(animationID)
	if (self.vars.animationList[animationID]) then
		return {
			order = animationID,
			type = 'text',
			name = self.vars.animationList[animationID].name,
			desc = L["NewAnimationCopy_Desc"],
			usage = L["NewAnimation_Usage"],
			get = function()
				return ""
			end,
			set = function(new)
				self:NewAnimation(self.vars.animationList[animationID], new)
			end,
			validate = function(new)
				return (strlen(new) > 0)
			end,
		}
	else
		return nil
	end
end


-------------------------------------------------------------------------------
-- Library - Test Frame
-------------------------------------------------------------------------------
function AvionAddin:ShowTestFrame(animationID)
	if (not self:IsTestFrameShown(animationID)) then
		-- Save link to frame that's beeing used
		self.vars.linkCache[animationID], _ = Avion:GetAvailableFrame()
		
		-- Save which triggerID triggerd this animation
		self.vars.linkCache[animationID].triggerID = animationID
		
		-- Convert to Avion-1.0 format
		local animationTable = self:AnimationInternToExtern(animationID)
		
		-- Show frame
		Avion:InitFrame(self.vars.linkCache[animationID], animationTable)
	end
end


-- Hide test frame (eg. if seting up options)
function AvionAddin:HideTestFrame(animationID)
	-- Create frame-memory
	if (self:IsTestFrameShown(animationID)) then
		local object = self.vars.linkCache[animationID]
		
		-- Hide frame and its not inUse anymore...
		object.texture:SetTexture("Interface\\Icons\\Temp.blp")
		object.frame:Hide()
		object.inUse = false
		
		-- Delete frame from animation cache
		self.vars.linkCache[animationID] = nil
	end
end


-- Is a frame shown
function AvionAddin:IsTestFrameShown(animationID)
	if (self.vars.linkCache[animationID]) then
		return true
	else
		return false
	end
end


-- Set animation-status of an animation to a certain time
function AvionAddin:SetTestFrameStatus(animationID, timer)
	-- Make sure frame is shown
	if (self:IsTestFrameShown(animationID)) then
		local object = self.vars.linkCache[animationID]
	
		-- Prepare table for Avion-1.0 usage
		local animationTable = self:AnimationInternToExtern(animationID)
		
		-- Reset Frame
		Avion:InitFrame(object, animationTable)

		-- Set timer
		object.elapsedTimes = timer/Avion.vars.updateInterval
		
		-- Update animation
		Avion:UpdateAnimation(object, animationTable, true)
		
		-- Reset timer
		object.elapsedTimes = timer/Avion.vars.updateInterval
	end
end


-- Get animation-status of an animation to a certain time
function AvionAddin:GetTestFrameStatus(animationID)
	-- Make sure frame is shown
	if (self:IsTestFrameShown(animationID)) then
		return self.vars.linkCache[animationID].elapsedTimes * Avion.vars.updateInterval
	else
		return 0
	end
end


-- Create a deep copy of an animationTable
function AvionAddin:DeepAnimationTableCopy(animationTable)
	return {
		name = animationTable.name,
		alpha = animationTable.alpha,
		color = {r = animationTable.color.r, g = animationTable.color.g, b = animationTable.color.b},
		position = {x = animationTable.position.x, y = animationTable.position.y},
		size = {width = animationTable.size.width, height = animationTable.size.height},
		rotation = animationTable.rotation,
		isAnimated = {color = animationTable.isAnimated.color, position = animationTable.isAnimated.position, scale = animationTable.isAnimated.scale, rotation = animationTable.isAnimated.rotation },
		animation = {color = {r = animationTable.animation.color.r, g = animationTable.animation.color.g, b = animationTable.animation.color.b}, position = {x = animationTable.animation.position.x, y = animationTable.animation.position.y}, scale = animationTable.animation.scale, rotation = animationTable.animation.rotation},
		animationFunc = {alpha = animationTable.animationFunc.alpha, color = animationTable.animationFunc.color, position = animationTable.animationFunc.position, scale = animationTable.animationFunc.scale, rotation = animationTable.animationFunc.rotation},
		addRandom = {alpha = animationTable.addRandom.alpha, color = animationTable.addRandom.color, position = animationTable.addRandom.position, scale = animationTable.addRandom.scale, rotation = animationTable.addRandom.rotation},
		animateAndFade = animationTable.animateAndFade,
		animationTime = animationTable.animationTime,
		fadeTime = animationTable.fadeTime,
		waitTime = animationTable.waitTime,
		showWhileWait = animationTable.showWhileWait,
		textureFile = animationTable.textureFile,
		soundFile = animationTable.soundFile,
	}
end


function AvionAddin:FunctionList()
	local list = {}
	
	for name, _ in pairs(self.vars.functionList) do 
		list[name] = name
	end
	
	return list
end


-- Create (real) function from SharedMedia/String entry
function AvionAddin:MediaToFunc(funcString)
	local func = assert(loadstring([[
		return function (param, rnd) 
			if (rnd) then
				return (]] .. (self.vars.functionList[funcString] or "0") ..[[ + math.random(-rnd, rnd) / 10000) 
			else 
				return (]] .. (self.vars.functionList[funcString] or "0") .. [[)
			end
		end
	]]))
	local result = func()
	
	return result
end


function AvionAddin:AnimationInternToExtern(animationID)
	-- Make a deep copy first
	local result = self:DeepAnimationTableCopy(self.vars.animationList[animationID])
	
	-- But we have to prepare the table a bit
	if (result.soundFile) then
		result.soundFile = Media:Fetch("sound", result.soundFile)
	else
		result.soundFile = nil
	end
	if (result.textureFile) then
		result.textureFile = Media:Fetch("aura", result.textureFile)
	else
		result.textureFile = "Interface\\Icons\\Temp.blp"
	end
	result.animationFunc = {
		alpha = self:MediaToFunc(result.animationFunc.alpha), 
		color = self:MediaToFunc(result.animationFunc.color), 
		position = self:MediaToFunc(result.animationFunc.position), 
		scale = self:MediaToFunc(result.animationFunc.scale), 
		rotation = self:MediaToFunc(result.animationFunc.rotation)
	}

	return result
end


-------------------------------------------------------------------------------
-- Library - API
-------------------------------------------------------------------------------
-- Register SavedVariables
function AvionAddin:RegisterAnimationDB(name)
	-- Register (own) Database
	self:RegisterDB(name)
	self:RegisterDefaults("AvionAddin-1.0", "profile", { 
		animations = {},
	})
	
	-- Link DB to addin
	self.vars.animationList = self:AcquireDBNamespace("AvionAddin-1.0").profile.animations
	
	self.vars.functionList = {
		["linear"] = "param",
		["square"] = "param*param",
		["squareroot"] = "math.sqrt(param)",
	}
	
	-- Update option table
	self:UpdateOptionTable()
end


-- Iterator that returns id and name for each animation
function AvionAddin:AnimationIterator()
	local id
	
	return function ()
		id = next(self.vars.animationList, id)

		-- we reached the end early?
		if id == nil then
			return nil
		end
		
		while self.vars.animationList[id] == nil do
			id = next(self.vars.animationList, id)
			
			if id == nil then
				return nil
			end
		end
		
		return id, self.vars.animationList[id].name
	end
end



-- Build Optionstable
function AvionAddin:UpdateOptionTable()
	-- Build option-table for each animation
	self.vars.OnMenuRequest.args.individual.args = nil
	self.vars.OnMenuRequest.args.individual.args = { }
	for animationID, _ in pairs(self.vars.animationList) do
		self.vars.OnMenuRequest.args.individual.args[animationID] = self:BuildAnimationOption(animationID)
	end
	
	-- Build a list of animations to copy data from
	self.vars.OnMenuRequest.args.new.args.copy.args = nil
	self.vars.OnMenuRequest.args.new.args.copy.args = { }
	for animationID, _ in pairs(self.vars.animationList) do
		self.vars.OnMenuRequest.args.new.args.copy.args[animationID] = self:BuildCopyOption(animationID)
	end
	
	-- Trigger Event
	self:TriggerEvent("AvionAddin_MenuUpdate")
	
	return self.vars.OnMenuRequest
end


-- Add new animation (from default-db or existing one)
function AvionAddin:NewAnimation(dataTable, newName)
	local animationID = #(self.vars.animationList) + 1
	
	-- Name should be a String and longer than zero chars
	if (strlen(newName) == 0) then
		return false
	end
	
	-- Default
	if (dataTable == nil) then
		self.vars.animationList[animationID] = {
			name = newName,
			alpha = 1,
			color = {r = 1, g = 1, b = 1},
			position = {x = 0, y = 0},
			size = {width = 320, height = 200},
			rotation = math.pi/2,
			isAnimated = {color = true, position = true, scale = true, rotation = true },
			animation = {color = {r = 0, g = 0, b = 1}, position = {x = 100, y = 0}, scale = 2, rotation = 0},
			animationFunc = {alpha = "linear", color = "linear", position = "linear", scale = "linear", rotation = "linear"},
			addRandom = {alpha = 100, color = 100, position = 100, scale = 100, rotation = 100},
			animateAndFade = false,
			animationTime = 3,
			fadeTime = 2,
			waitTime = 0,
			showWhileWait = true,
			textureFile = "Cross",
			soundFile = "Boing",
		}
	-- Existing
	else
		self.vars.animationList[animationID] = self:DeepAnimationTableCopy(dataTable)
		self.vars.animationList[animationID].name = newName
	end
	
	-- Refresh 
	self:UpdateOptionTable()
	
	-- Trigger Event
	self:TriggerEvent("AvionAddin_NewAnimation", animationID)
	
	return animationID
end


-- Delete animation from db
function AvionAddin:DelAnimation(animationID)
	-- Stop running animations and delete
	self:StopAllAnimations()
	self.vars.animationList[animationID] = nil

	-- Refresh
	self:UpdateOptionTable()
	
	-- Trigger Event
	self:TriggerEvent("AvionAddin_DeleteAnimation", animationID)
	
	return animationID
end


-- Assosiate a lis of animations (settings) with the options
function AvionAddin:FeedAnimationList(animationList)
	-- Clean first
	self.vars.animationList = nil
	self.vars.animationList = { }
	
	-- Deep copy of each animation
	for _, data in pairs(animationList) do
		self:NewAnimation(dataTable, dataTable.name)
	end
end


-- Trigger Animation with certain animationID
function AvionAddin:TriggerAnimationByID(triggerID, animationID)
	-- Make a deep copy
	local animationTable = self:AnimationInternToExtern(animationID)

	-- Trigger it
	self:TriggerAnimation(triggerID, animationTable)
end


-- !!! This should be removed once really embedding !!!
function AvionAddin:TriggerAnimation(triggerID, animationTable)	
	Avion:TriggerAnimation(triggerID, animationTable)
end
function AvionAddin:StopAnimation(triggerID)
	Avion:StopAnimation(triggerID)
end
function AvionAddin:StopAllAnimations()
	Avion:StopAllAnimations()
end
function AvionAddin:IsAnimationRunning(triggerID)
	Avion:IsAnimationRunning(triggerID)
end


-------------------------------------------------------------------------------
-- Activation function
-------------------------------------------------------------------------------
-- Activate a new instance of this library
local function activate(self, oldLib, oldDeactivate)
	-- If there is an old library running, disable and copy everything
	if oldLib then
		-- Copy vars from old library
		self.vars = oldLib.vars
	else
		self.vars = {}
		self.vars.animationList = {}
		self.vars.testAnimationCache = {}
		self.vars.linkCache = { }
	end

	-- (Re-) initialize variables
	self.vars.running 		= true
	Media 					= AceLibrary("LibSharedMedia-2.0")
	Avion					= AceLibrary("Avion-1.0")
	
	-- Update Table
	self.vars.OnMenuRequest = self:BuildDefaultOptions()
	self:UpdateOptionTable()

	if oldDeactivate then oldDeactivate(oldLib) end
end


-------------------------------------------------------------------------------
-- Register
-------------------------------------------------------------------------------
AceLibrary:Register(AvionAddin, MAJOR_VERSION, MINOR_VERSION, activate)
AvionAddin = nil
